import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:path_provider/path_provider.dart';
import '../value.dart';

class StorageImpl {
  StorageImpl(this.fileName, [this.path]);

  final String path, fileName;

  final Value<Map<String, dynamic>> subject =
      Value<Map<String, dynamic>>(<String, dynamic>{});

  File _file;

  Future<void> clear() async {
    //  File _file = await _getFile();
    subject
      ..value.clear()
      ..update("", 2, null);
    subject.value.clear();
    return _file.deleteSync();
  }

  // Future<bool> _exists() async {
  //   File _file = await _getFile();
  //   return _file.existsSync();
  // }

  Future<void> flush() async {
    final serialized = json.encode(subject.value);
    //  File _file = await _getFile();
    await _file.writeAsString(serialized, flush: true);
    return;
  }

  T read<T>(String key) {
    return subject.value[key] as T;
  }

  Future<void> init([Map<String, dynamic> initialData]) async {
    subject.value = initialData ?? <String, dynamic>{};
    _file = await _getFile();
    if (_file.existsSync()) {
      return _readFile();
    } else {
      return _writeFile(subject.value);
    }
  }

  Future<void> remove(String key) async {
    subject
      ..value.remove(key)
      ..update(key, 1, null);
    await _writeFile(subject.value);
  }

  Future<void> write(String key, dynamic value) async {
    writeInMemory(key, value);
    await _writeFile(subject.value);
  }

  void writeInMemory(String key, dynamic value) {
    subject
      ..value[key] = value
      ..update(key, 0, value);
  }

  Future<void> _writeFile(Map<String, dynamic> data) async {
    // File _file = await _getFile();
    _file.writeAsString(json.encode(data), flush: true);
  }

  Future<void> _readFile() async {
    // File _file = await _getFile();
    final content = await _file.readAsString()
      ..trim();
    try {
      subject.value = json?.decode(content) as Map<String, dynamic>;
    } catch (e) {
      rethrow;
    }
  }

  Future<File> _getFile() async {
    final dir = await _getDocumentDir();
    final _path = path ?? dir.path;
    final _file = File('$_path/$fileName.gs');
    return _file;
  }

  Future<Directory> _getDocumentDir() async {
    try {
      return await getApplicationSupportDirectory();
    } catch (err) {
      throw err;
    }
  }
}
